#ifndef __SHELLHANDLER_H
#define __SHELLHANDLER_H

#include <configuratorsShellFE.h> // for ShellFEType
#include <legacyVtkWriter.h>

namespace shellFE {

enum ShellBoundaryType {
      ALLBOUNDARY = 100,
      NOBOUNDARY = 0,
  //Plate
      PlateLeft = 1,
      PlateLeftTop = 2,
      PlateAll = 3,
};

//! \brief Additional information about TriangleMeshes
template< typename ConfiguratorType >
class ShellHandler {
public:
  typedef typename ConfiguratorType::RealType       RealType;
  typedef typename ConfiguratorType::InitType       MeshType;
  typedef typename ConfiguratorType::MaskType       MaskType;
  typedef typename ConfiguratorType::TangentVecType TangentVecType;
  typedef typename ConfiguratorType::Point3DType    Point3DType;
  typedef typename ConfiguratorType::VectorType     VectorType;
  
  const ConfiguratorType &_conf;
  const MeshType &_mesh;
  const int _numVertices, _numGlobalDofs;
  MaskType _DirichletMask;
  int _numBoundaryNodes;
  mutable VectorType _xA;
  
public:
  
  ShellHandler( const ConfiguratorType &conf ) :
  _conf ( conf ),
  _mesh( conf.getInitializer() ),
  _numVertices ( _mesh.getNumVertices() ),
  _numGlobalDofs ( conf.getNumGlobalDofs() ),
  _xA ( 3 * conf.getNumGlobalDofs() )
  {
    generateChart_xA ( );
    generateDirichletBoundaryMask( _DirichletMask, _numBoundaryNodes );
  }

  void generateChart_xA() const
  {
	  for ( int nodeIdx=0; nodeIdx < _numVertices; ++nodeIdx ) {
		  const Point3DType& coords ( _mesh.getVertex(nodeIdx) );

		  for( int comp=0; comp<3; ++comp )
			  _xA[nodeIdx + _numGlobalDofs * comp] = coords[comp];
	  }
  }

  VectorType& getChartToUndeformedShell() const
  {
	  return _xA;
  }
  
  void generateDirichletBoundaryMask ( MaskType & mask, int & numBoundaryNodes,
		  ShellBoundaryType boundaryType = ALLBOUNDARY,
		  bool clampedBoundaryCondition = false) const{
    mask.resize( _numGlobalDofs, false );
    numBoundaryNodes = 0;
    
    switch ( boundaryType ){
      case NOBOUNDARY : {
      } break;
      
      case ALLBOUNDARY : {
            for( int ElementIndex = 0; ElementIndex < _mesh.getNumTriangs(); ++ElementIndex ){
                for( int i = 0; i < 3; i++ ){
                    if ( _mesh.getNeighbour( ElementIndex , i ) == -1 ){
                        mask[ _mesh.getTriangNodeIdx( ElementIndex, (i+1)%3) ] = true;
                        mask[ _mesh.getTriangNodeIdx( ElementIndex, (i+2)%3) ] = true;
                    }
                }
            }
      } break;
      
      case PlateLeft : {
            for ( int nodeIdx=0; nodeIdx < _mesh.getNumVertices(); ++nodeIdx ) {
              const Point3DType& coords ( _mesh.getVertex(nodeIdx) );
              if (coords [0] == 0. ){
                ++numBoundaryNodes;
                mask[nodeIdx] = true;
              }
            }
      } break;
      
      case PlateLeftTop : {
            for ( int nodeIdx=0; nodeIdx < _mesh.getNumVertices(); ++nodeIdx ) {
              const Point3DType& coords ( _mesh.getVertex(nodeIdx) );
              if (coords [0] == 0. || coords[1] == 1. ){
                ++numBoundaryNodes;
                mask[nodeIdx] = true;
              }
            }
      } break;
      
     case PlateAll : {
            for ( int nodeIdx=0; nodeIdx < _mesh.getNumVertices(); ++nodeIdx ) {
              const Point3DType& coords ( _mesh.getVertex(nodeIdx) );
              if (coords [0] == 0. || coords [0] == 1. || coords [1] == 0. || coords[1] == 1. ){
                ++numBoundaryNodes;
                mask[nodeIdx] = true;
              }
            }
      } break;

      
      default :
        throw std::invalid_argument( aol::strprintf ( "Wrong boundary condition. In File %s at line %d.", __FILE__, __LINE__ ).c_str() );
        break;
    }
    
    //clamped boundary condition
    if( (ConfiguratorType::_ShellFEType == C1Dofs) && (clampedBoundaryCondition) ){
        for( int i=0; i<_numVertices; ++i ){
          if( mask[i] ){
            mask[ i + _numVertices] = true;
            mask[ i + 2. * _numVertices] = true;
          }
        }
    }
    
  }
  
  const MaskType & getDirichletMask ( ) const { return _DirichletMask;}
  const int & getNumBoundaryNodes() const { return _numBoundaryNodes; }
};

} //end namespace


#endif //__SHANDLER_H
